import logging
from typing import List

import transaction

from workspaces.capability.helpers import CapabilitySequence
from workspaces.capability.schema_interfaces import CapabilityExecutionIF
from workspaces.capability.services.interfaces import (
    CapabilityEngineIF,
    CapabilityInfoIF,
)
from workspaces.system.schema import AbstractFile
from workspaces.workflow.services.interfaces import WorkflowServiceIF

logger = logging.getLogger(__name__)


class CapabilityEngine(CapabilityEngineIF):
    """
    Executes a prepare and run workflow step of a capability
    """

    def __init__(
        self,
        capability_info: CapabilityInfoIF,
        workflow_service: WorkflowServiceIF,
    ):
        self.capability_info = capability_info
        self.workflow_service = workflow_service
        self.execution = None

    def load_engine(self, execution: CapabilityExecutionIF):
        """
        Load capability execution of interest into capability engine for access and execution
        :param execution: execution to be executed by this engine
        """
        self.execution = execution

    def unload_engine(self):
        """
        Unload capability engine to free it up for future executions
        """
        self.execution = None

    def execute(self):
        """
        Communicate with workflow service and send a request to run a workflow with given settings
        """
        step_sequence = CapabilitySequence.from_str(self.execution.steps)
        # Grab value of current step (workflow name)
        cur_step = step_sequence[int(self.execution.current_step)]
        cur_step.execute(self, self.execution)

    def submit_workflow_request(
        self,
        execution_id: int,
        workflow_name: str,
        workflow_args: dict,
        files: List[AbstractFile],
    ):
        """
        Submit a workflow request to the workflow service

        :param execution_id:
        :param workflow_name:
        :param workflow_args:
        :param files:
        """
        transfer_files_string = self.get_transfer_files_property(files)

        args_list = {**workflow_args, "files_to_transfer": transfer_files_string}

        logger.info(
            "creating workflow request with name %s, args %s (and arg type %s)",
            workflow_name,
            args_list,
            type(args_list),
        )
        workflow_request = self.workflow_service.create_workflow_request(
            workflow=workflow_name, argument=args_list
        )

        logger.info(
            "My workflow_request is %s, with type %s and workflow request ID %s",
            workflow_request,
            type(workflow_request),
            workflow_request.workflow_request_id,
        )

        for file in files:
            response = self.workflow_service.attach_file_to_request(
                workflow_request, file.filename, file.content
            )
            logger.info(
                f"Added file {file.filename} to workflow request #{workflow_request.workflow_request_id}"
            )

        execution = self.capability_info.lookup_execution(execution_id)
        execution.update_execution(workflow_request.workflow_request_id)
        self.capability_info.save_execution(execution)
        transaction.commit()
        self.workflow_service.execute(workflow_request)

    def announce_qa_ready(self, execution_id: int):
        execution = self.capability_info.lookup_execution(execution_id)
        self.workflow_service.announce_qa_ready(execution.current_workflow_request_id)

    @staticmethod
    def get_transfer_files_property(files: List[AbstractFile]):
        separator = ", ./"
        return separator + separator.join(file.filename for file in files)
